package com.stellvoy.user.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.stellvoy.api.client.auth.AuthClient;
import com.stellvoy.api.client.task.TaskClient;
import com.stellvoy.api.dto.task.NewTaskDTO;
import com.stellvoy.api.dto.user.LoginFormDTO;
import com.stellvoy.common.domain.dto.LoginUserDTO;
import com.stellvoy.common.exceptions.BadRequestException;
import com.stellvoy.common.exceptions.ForbiddenException;
import com.stellvoy.common.utils.AssertUtils;
import com.stellvoy.common.utils.BeanUtils;
import com.stellvoy.common.utils.StringUtils;
import com.stellvoy.common.utils.UserContext;
import com.stellvoy.user.domain.dto.*;
import com.stellvoy.user.domain.po.User;
import com.stellvoy.user.domain.po.UserTask;
import com.stellvoy.user.mapper.UserMapper;
import com.stellvoy.user.service.ICodeService;
import com.stellvoy.user.service.IUserService;
import com.stellvoy.user.service.UserTaskService;
import com.stellvoy.user.utils.AliyunIdCardVerifier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;


import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.stellvoy.user.constants.UserErrorInfo.Msg.*;


/**
 * <p>
 * 学员用户表 服务实现类
 * </p>
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private ICodeService codeService;
    @Autowired
    private AliyunIdCardVerifier aliyunIdCardVerifier;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private UserTaskService userTaskService;
    @Autowired
    private TaskClient taskClient;
    @Override
    public LoginUserDTO queryUserDetail(LoginFormDTO loginDTO, boolean isStaff) {
        // 1.判断登录方式
        Integer type = loginDTO.getType();
        User user = null;
        // 2.手机号和密码登录
        if (type == 1) {
            user = loginByPw(loginDTO);
        }
        // 3.验证码登录
        if (type == 2) {
            user = loginByVerifyCode(loginDTO.getCellPhone(), loginDTO.getPassword());
        }
        // 4.错误的登录方式
        if (user == null) {
            throw new BadRequestException(ILLEGAL_LOGIN_TYPE);
        }
        // 6.封装返回
        LoginUserDTO userDTO = new LoginUserDTO();
        userDTO.setUserId(user.getId());
        userDTO.setRoleId(handleRoleId(user));
        return userDTO;
    }

    /**
     * 手机号注册
     */
    public User phoneRegister(UserRegisterDTO dto) {
        // 1.校验验证码
        codeService.verifyCode(dto.getCellPhone(), dto.getCode());
        // 2.根据手机号查询
        User one = lambdaQuery().eq(User::getCellPhone, dto.getCellPhone()).one();
        if (one != null) {
            throw new BadRequestException("手机号已经被注册");
        }
        User user = BeanUtils.copyBean(dto, User.class);
        user.setStatus(1);
//        user.setPassword(passwordEncoder.encode(dto.getPassword()));
        user.setPassword("123456");
        user.setCreateTime(LocalDateTime.now());
        user.setUpdateTime(LocalDateTime.now());
        user.setCreater(UserContext.getUser());
        user.setUpdater(UserContext.getUser());
        user.setUsername(dto.getCellPhone());
        user.setImage("https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/woman.png");
        save(user);
        return user;
    }

    /**
     * 实名认证
     */
    public void verifyCode(VerifyCodeDTO dto, Long userId) {
        // 验证身份证
        if (StringUtils.isNotBlank(dto.getIdentityCardId())) {
            if (!aliyunIdCardVerifier.verifyIdentityCard(dto.getIdentityCardId(), dto.getName())) {
                throw new RuntimeException("身份证验证失败");
            }
        } else {
            throw new RuntimeException("身份证号码不能为空");
        }
        int sex = 0;
        String image = "https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/woman.png";
        if(!sex(dto.getIdentityCardId())){
            sex = 1;
            image = "https://suppliermanage.oss-cn-shanghai.aliyuncs.com/gig/img/man.png";
        }
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getIdentityCardId, dto.getIdentityCardId())
                .set(User::getSex, sex)
                .set(User::getImage, image)
                .update();
    }
    private boolean sex(String getIdentityCardId) {
        char[] charArray = getIdentityCardId.toCharArray();
        return charArray[16] % 2 == 0;
    }
    /**
     * 修改登录密码
     */
    public void updatePassword(UpdatePasswordDTO dto, Long userId) {
        User user = getById(userId);
        if (user == null) {
            throw new BadRequestException(USER_ID_NOT_EXISTS);
        }
        // 1.校验旧密码
        if (!user.getPassword().equals(dto.getOldPassword())) {
            throw new BadRequestException("密码错误");
        }
        // 2.修改密码
        user.setPassword(dto.getNewPassword());
        updateById(user);
    }

    /**
     * 绑定邮箱
     */
    public void updateEmail(UpdateEmailDTO dto, Long userId) {
        // 从Redis中获取验证码
        String storedCode = stringRedisTemplate.opsForValue().get("verificationCode:" + dto.getEmail());
        // 验证码校验
        if (storedCode == null || !storedCode.equals(dto.getCode())) {
            throw new RuntimeException("验证码错误或已过期");
        }
        // 验证码正确，清空Redis中的验证码
        stringRedisTemplate.delete("verificationCode:" + dto.getEmail());
        lambdaUpdate()
                .eq(User::getId, userId)
                .set(User::getEmail, dto.getEmail())
                .update();
    }

    /**
     * 统计用户任务数量
     */
    public UserTaskDTO getStats(Long userId) {
        List<UserTask> userTaskList = userTaskService.lambdaQuery().eq(UserTask::getUserId, userId).list();

        // 检查 userTaskList 是否为空
        if (userTaskList == null || userTaskList.isEmpty()) {
            return new UserTaskDTO();
        }

        UserTaskDTO userTaskDTO = new UserTaskDTO();
        userTaskDTO.setOngoingTasks(0);
        userTaskDTO.setSettledTasks(0);

        Map<Integer, TaskMonth> tasks2024 = new HashMap<>();
        for (int i = 1; i <= 12; i++) {
            TaskMonth taskMonth = new TaskMonth();
            taskMonth.setTasksReceived(0);
            taskMonth.setTasksCompleted(0);
            tasks2024.put(i, taskMonth);
        }

        for (UserTask userTask : userTaskList) {
            // 获取任务创建月份
            LocalDateTime taskDateTime = userTask.getCreateTime();
            int month = taskDateTime.getMonthValue();

            if (userTask.getTaskStatus() == 2) {
                userTaskDTO.setOngoingTasks(userTaskDTO.getOngoingTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
            } else if (userTask.getTaskStatus() == 3) {
                userTaskDTO.setSettledTasks(userTaskDTO.getSettledTasks() + 1);
                tasks2024.get(month).setTasksReceived(tasks2024.get(month).getTasksReceived() + 1);
                tasks2024.get(month).setTasksCompleted(tasks2024.get(month).getTasksCompleted() + 1);
            }
        }
        userTaskDTO.setClaimedTasks(userTaskDTO.getOngoingTasks() + userTaskDTO.getSettledTasks());
        userTaskDTO.setTasks2024(tasks2024);

        List<Long> ids = userTaskList.stream()
                .filter(userTask -> userTask.getTaskStatus() == 3)
                .map(UserTask::getTaskId)
                .collect(Collectors.toList());

        // 检查 ids 是否为空
        if (ids.isEmpty()) {
            userTaskDTO.setTaskDetails(new ArrayList<>());
            return userTaskDTO;
        }

        List<NewTaskDTO> newTaskDTOS = taskClient.queryTaskById(ids);

        // 检查 newTaskDTOS 是否为空
        if (newTaskDTOS == null || newTaskDTOS.isEmpty()) {
            userTaskDTO.setTaskDetails(new ArrayList<>());
        } else {
            List<TaskDetail> taskDetails = BeanUtils.copyList(newTaskDTOS, TaskDetail.class);
            userTaskDTO.setTaskDetails(taskDetails);
        }

        return userTaskDTO;
    }

    public User loginByPw(LoginFormDTO loginDTO) {
        // 1.数据校验
        String cellPhone = loginDTO.getCellPhone();
        if (StrUtil.isBlank(cellPhone)) {
            throw new BadRequestException(INVALID_UN);
        }
        // 2.根据手机号查询
        User user = lambdaQuery()
                .eq(StrUtil.isNotBlank(cellPhone), User::getCellPhone, cellPhone)
                .one();
        AssertUtils.isNotNull(user, INVALID_UN_OR_PW);
        // 3.校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        // 4.校验密码
//        if (!passwordEncoder.matches(loginDTO.getPassword(), user.getPassword())) {
//            throw new BadRequestException(INVALID_UN_OR_PW);
//        }
        if (!loginDTO.getPassword().equals(user.getPassword())){
            throw new BadRequestException(INVALID_UN_OR_PW);
        }

        return user;
    }

    private Long handleRoleId(User user) {
        return 0L;
    }

    public User loginByVerifyCode(String phone, String code) {
        User user = lambdaQuery().eq(User::getCellPhone, phone).one();
        if (user == null) {
            UserRegisterDTO URDTO = new UserRegisterDTO();
            URDTO.setCode(code);
            URDTO.setCellPhone(phone);
            return phoneRegister(URDTO);
        }
        // 校验验证码
        codeService.verifyCode(phone, code);
        // 校验是否禁用
        if (user.getStatus() == 0) {
            throw new ForbiddenException(USER_FROZEN);
        }
        return user;
    }
}
